"""Authentication client for songbox API"""

from typing import Dict, Any, Optional
from .exceptions import AuthenticationError, ValidationError


class AuthClient:
    """Client for handling authentication with the songbox API.
    
    This client handles login, OTP verification, and token refresh operations.
    """
    
    def __init__(self, client):
        self._client = client
    
    def login(self, country_code: str, mobile_number: str) -> str:
        """Initiate login process by sending OTP to mobile number.
        
        Args:
            country_code: Country code (e.g., "960" for Maldives)
            mobile_number: Mobile number without country code
            
        Returns:
            verify_id: ID needed for OTP verification
            
        Raises:
            ValidationError: If input parameters are invalid
            AuthenticationError: If login initiation fails
            
        Example:
            >>> auth = AuthClient(client)
            >>> verify_id = auth.login("960", "7777777")
        """
        if not country_code or not mobile_number:
            raise ValidationError("Country code and mobile number are required")
        
        data = {
            "country_code": country_code,
            "mobile_number": mobile_number
        }
        
        try:
            response = self._client.request(
                method="POST",
                endpoint="/v2/auth/init",
                data=data,
                require_auth=False
            )
            
            verify_id = response.get("verify_id")
            if not verify_id:
                raise AuthenticationError("Failed to get verification ID")
            
            return verify_id
            
        except Exception as e:
            if isinstance(e, (AuthenticationError, ValidationError)):
                raise
            raise AuthenticationError(f"Login failed: {str(e)}")
    
    def verify_otp(self, otp: str, verify_id: str) -> str:
        """Verify OTP and get authentication token.
        
        Args:
            otp: One-time password received via SMS
            verify_id: Verification ID from login step
            
        Returns:
            token: JWT authentication token
            
        Raises:
            ValidationError: If input parameters are invalid
            AuthenticationError: If OTP verification fails
            
        Example:
            >>> token = auth.verify_otp("123456", verify_id)
            >>> client.set_token(token)
        """
        if not otp or not verify_id:
            raise ValidationError("OTP and verify_id are required")
        
        data = {
            "otp": otp,
            "verify_id": verify_id
        }
        
        try:
            response = self._client.request(
                method="POST",
                endpoint="/v2/sign_in?auth_method=otp",
                data=data,
                require_auth=False
            )
            
            token = response.get("token")
            if not token:
                raise AuthenticationError("Failed to get authentication token")
            
            # Automatically set the token in the client
            self._client.set_token(token)
            
            return token
            
        except Exception as e:
            if isinstance(e, (AuthenticationError, ValidationError)):
                raise
            raise AuthenticationError(f"OTP verification failed: {str(e)}")
    
    def refresh_token(self) -> str:
        """Refresh the current authentication token.
        
        Returns:
            token: New JWT authentication token
            
        Raises:
            AuthenticationError: If token refresh fails
            
        Example:
            >>> new_token = auth.refresh_token()
        """
        try:
            response = self._client.request(
                method="GET",
                endpoint="/v1/refresh_token",
                require_auth=True
            )
            
            token = response.get("token")
            if not token:
                raise AuthenticationError("Failed to refresh token")
            
            # Update the token in the client
            self._client.set_token(token)
            
            return token
            
        except Exception as e:
            if isinstance(e, AuthenticationError):
                raise
            raise AuthenticationError(f"Token refresh failed: {str(e)}")
    
    def logout(self) -> None:
        """Logout by clearing the authentication token.
        
        Example:
            >>> auth.logout()
        """
        self._client.clear_token()
    
    def get_token_info(self) -> Optional[Dict[str, Any]]:
        """Get information about the current token.
        
        Returns:
            Token information including expiry date, or None if not authenticated
            
        Example:
            >>> info = auth.get_token_info()
            >>> if info:
            ...     print(f"Token expires: {info.get('expire')}")
        """
        if not self._client.is_authenticated:
            return None
        
        try:
            response = self._client.request(
                method="GET",
                endpoint="/v1/refresh_token",
                require_auth=True
            )
            
            return {
                "token": response.get("token"),
                "expire": response.get("expire"),
                "code": response.get("code")
            }
            
        except Exception:
            return None