"""Main songbox API Client"""

import httpx
from typing import Optional, Dict, Any
from .auth import AuthClient
from .user import UserClient
from .music import MusicClient
from .exceptions import songboxError, APIError


class SongboxClient:
    """Main client for interacting with the songbox API.
    
    This client provides access to all songbox API endpoints through
    modular sub-clients for authentication, user management, and music.
    
    Args:
        base_url: The base URL for the songbox API
        timeout: Request timeout in seconds (default: 30)
        headers: Additional headers to include with requests
    
    Example:
        >>> client = SongboxClient()
        >>> # Authenticate
        >>> verify_id = client.auth.login("960", "7777777")
        >>> token = client.auth.verify_otp("123456", verify_id)
        >>> client.set_token(token)
        >>> # Get user info
        >>> user_info = client.user.get_me()
        >>> print(user_info)
    """
    
    def __init__(
        self,
        base_url: str = "https://lavafoshi.online",
        timeout: float = 30.0,
        headers: Optional[Dict[str, str]] = None
    ):
        self.base_url = base_url.rstrip('/')
        self._token: Optional[str] = None
        
        # Default headers
        default_headers = {
            "User-Agent": "songbox/2.1.4 (songbox; Python) httpx",
            "Accept": "*/*",
            "Accept-Language": "en-US;q=1.0",
            "Accept-Encoding": "gzip;q=1.0, compress;q=0.5",
            "Connection": "keep-alive"
        }
        
        if headers:
            default_headers.update(headers)
        self._http_client = httpx.Client(
            base_url=self.base_url,
            timeout=timeout,
            headers=default_headers,
            follow_redirects=True
        )
        self.auth = AuthClient(self)
        self.user = UserClient(self)
        self.music = MusicClient(self)
    
    def set_token(self, token: str) -> None:
        """Set the authentication token for API requests.
        
        Args:
            token: JWT token obtained from authentication
        """
        self._token = token
        self._http_client.headers["Authorization"] = f"Bearer {token}"
    
    def clear_token(self) -> None:
        """Clear the authentication token."""
        self._token = None
        if "Authorization" in self._http_client.headers:
            del self._http_client.headers["Authorization"]
    
    @property
    def token(self) -> Optional[str]:
        """Get the current authentication token."""
        return self._token
    
    @property
    def is_authenticated(self) -> bool:
        """Check if the client is authenticated."""
        return self._token is not None
    
    def request(
        self,
        method: str,
        endpoint: str,
        params: Optional[Dict[str, Any]] = None,
        data: Optional[Dict[str, Any]] = None,
        json: Optional[Dict[str, Any]] = None,
        headers: Optional[Dict[str, str]] = None,
        require_auth: bool = True
    ) -> Dict[str, Any]:
        """Make an HTTP request to the songbox API.
        
        Args:
            method: HTTP method (GET, POST, etc.)
            endpoint: API endpoint (without base URL)
            params: Query parameters
            data: Form data for POST requests
            json: JSON data for POST requests
            headers: Additional headers
            require_auth: Whether authentication is required
            
        Returns:
            JSON response data
            
        Raises:
            songboxError: If authentication is required but not provided
            APIError: If the API returns an error response
        """
        if require_auth and not self.is_authenticated:
            raise songboxError("Authentication required for this endpoint")
        
        url = endpoint if endpoint.startswith('http') else f"/api{endpoint}"
        
        try:
            response = self._http_client.request(
                method=method,
                url=url,
                params=params,
                data=data,
                json=json,
                headers=headers
            )
            
            response.raise_for_status()
            
            try:
                return response.json()
            except ValueError:
                return {}
                
        except httpx.HTTPStatusError as e:
            try:
                error_data = e.response.json()
                message = error_data.get('message', f'HTTP {e.response.status_code}')
            except ValueError:
                message = f'HTTP {e.response.status_code}'
            
            raise APIError(message, status_code=e.response.status_code)
        
        except httpx.RequestError as e:
            raise songboxError(f"Request failed: {str(e)}")
    
    def close(self) -> None:
        """Close the HTTP client."""
        self._http_client.close()
    
    def __enter__(self):
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()