"""Music client for songbox API"""

import os
import httpx
from pathlib import Path
from urllib.parse import quote
from typing import Dict, Any, List, Optional
from .exceptions import AuthenticationError, NotFoundError, ValidationError


class MusicClient:
    """Client for handling music-related operations with the songbox API.
    
    This client handles search, albums, songs, and other music operations.
    """
    
    def __init__(self, client):
        self._client = client
    
    def search(self, query: str) -> List[Dict[str, Any]]:
        """Search for songs, artists, and albums.
        
        Args:
            query: Search query string
            
        Returns:
            List of search results categorized by type (Songs, Artists, Albums)
            
        Raises:
            ValidationError: If query is empty
            AuthenticationError: If not authenticated
            
        Example:
            >>> results = client.music.search("huttaa")
            >>> for category in results:
            ...     print(f"Category: {category['title']}")
            ...     for item in category['items']:
            ...         print(f"  - {item['heading']}")
        """
        if not query or not query.strip():
            raise ValidationError("Search query cannot be empty")
        
        try:
            response = self._client.request(
                method="GET",
                endpoint=f"/v1/search/{query.strip()}",
                require_auth=True
            )
            
            if isinstance(response, list):
                return response
            elif isinstance(response, dict):
                return response.get('results', response.get('data', []))
            else:
                return []
                
        except Exception as e:
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise AuthenticationError("Authentication required or token expired")
            raise
    
    def get_album(self, album_id: str) -> Dict[str, Any]:
        """Get detailed information about an album.
        
        Args:
            album_id: ID of the album
            
        Returns:
            Album information including songs, artist, artwork, etc.
            
        Raises:
            ValidationError: If album_id is empty
            AuthenticationError: If not authenticated
            NotFoundError: If album not found
            
        Example:
            >>> album = client.music.get_album("808")
            >>> print(f"Album: {album['name']} by {album['artist_name']}")
            >>> print(f"Songs: {len(album['songs'])}")
        """
        if not album_id or not str(album_id).strip():
            raise ValidationError("Album ID cannot be empty")
        
        try:
            response = self._client.request(
                method="GET",
                endpoint=f"/v1/albums/{album_id}",
                require_auth=True
            )
            
            return response
            
        except Exception as e:
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise AuthenticationError("Authentication required or token expired")
            elif "404" in str(e) or "not found" in str(e).lower():
                raise NotFoundError(f"Album with ID {album_id} not found")
            raise
    
    def get_song(self, song_id: str) -> Dict[str, Any]:
        """Get detailed information about a song.
        
        Args:
            song_id: ID of the song
            
        Returns:
            Song information including artist, album, duration, URLs, etc.
            
        Raises:
            ValidationError: If song_id is empty
            AuthenticationError: If not authenticated
            NotFoundError: If song not found
            
        Example:
            >>> song = client.music.get_song("6200")
            >>> print(f"Song: {song['name']} by {song['artist_name']}")
            >>> print(f"Duration: {song['duration']} seconds")
        """
        if not song_id or not str(song_id).strip():
            raise ValidationError("Song ID cannot be empty")
        
        try:
            response = self._client.request(
                method="GET",
                endpoint=f"/v1/songs/{song_id}",
                require_auth=True
            )
            
            return response
            
        except Exception as e:
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise AuthenticationError("Authentication required or token expired")
            elif "404" in str(e) or "not found" in str(e).lower():
                raise NotFoundError(f"Song with ID {song_id} not found")
            raise
    
    def get_artist(self, artist_id: str) -> Dict[str, Any]:
        """Get detailed information about an artist.
        
        Args:
            artist_id: ID of the artist
            
        Returns:
            Artist information including albums, songs, etc.
            
        Raises:
            ValidationError: If artist_id is empty
            AuthenticationError: If not authenticated
            NotFoundError: If artist not found
            
        Example:
            >>> artist = client.music.get_artist("124")
            >>> print(f"Artist: {artist['name']}")
        """
        if not artist_id or not str(artist_id).strip():
            raise ValidationError("Artist ID cannot be empty")
        
        try:
            response = self._client.request(
                method="GET",
                endpoint=f"/v1/artists/{artist_id}",
                require_auth=True
            )
            
            return response
            
        except Exception as e:
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise AuthenticationError("Authentication required or token expired")
            elif "404" in str(e) or "not found" in str(e).lower():
                raise NotFoundError(f"Artist with ID {artist_id} not found")
            raise
    
    def get_recommendations(self, limit: Optional[int] = None) -> List[Dict[str, Any]]:
        """Get personalized music recommendations.
        
        Args:
            limit: Maximum number of items to return
            
        Returns:
            List of recommended music items
            
        Raises:
            AuthenticationError: If not authenticated
            
        Example:
            >>> recommendations = client.music.get_recommendations(limit=5)
            >>> for item in recommendations:
            ...     print(f"Recommended: {item.get('name', 'Unknown')}")
        """
        params = {}
        if limit is not None:
            params['limit'] = limit
        
        try:
            response = self._client.request(
                method="GET",
                endpoint="/v1/recommendations",
                params=params,
                require_auth=True
            )
            
            if isinstance(response, list):
                return response
            elif isinstance(response, dict):
                return response.get('data', response.get('items', []))
            else:
                return []
                
        except Exception as e:
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise AuthenticationError("Authentication required or token expired")
            raise
    
    def get_trending(self, limit: Optional[int] = None) -> List[Dict[str, Any]]:
        """Get trending music.
        
        Args:
            limit: Maximum number of items to return
            
        Returns:
            List of trending music items
            
        Raises:
            AuthenticationError: If not authenticated
            
        Example:
            >>> trending = client.music.get_trending(limit=10)
            >>> for item in trending:
            ...     print(f"Trending: {item.get('name', 'Unknown')}")
        """
        params = {}
        if limit is not None:
            params['limit'] = limit
        
        try:
            response = self._client.request(
                method="GET",
                endpoint="/v1/trending",
                params=params,
                require_auth=True
            )
            
            if isinstance(response, list):
                return response
            elif isinstance(response, dict):
                return response.get('data', response.get('items', []))
            else:
                return []
                
        except Exception as e:
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise AuthenticationError("Authentication required or token expired")
            raise
    
    def get_playlists(self) -> List[Dict[str, Any]]:
        """Get user's playlists.
        
        Returns:
            List of user's playlists
            
        Raises:
            AuthenticationError: If not authenticated
            
        Example:
            >>> playlists = client.music.get_playlists()
            >>> for playlist in playlists:
            ...     print(f"Playlist: {playlist.get('name', 'Unknown')}")
        """
        try:
            response = self._client.request(
                method="GET",
                endpoint="/v1/playlists",
                require_auth=True
            )
            
            if isinstance(response, list):
                return response
            elif isinstance(response, dict):
                return response.get('data', response.get('playlists', []))
            else:
                return []
                
        except Exception as e:
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise AuthenticationError("Authentication required or token expired")
            raise
    
    def download_song(self, song_url: str, download_path: Optional[str] = None) -> str:
        """Download a song from the given URL.
        
        Args:
            song_url: The song URL (typically from song['url_original'])
            download_path: Optional download directory path. If not provided, uses './downloads'
            
        Returns:
            Path to the downloaded file
            
        Raises:
            ValidationError: If song_url is empty
            Exception: If download fails
            
        Example:
            >>> song = client.music.get_song("6200")
            >>> file_path = client.music.download_song(song['url_original'])
            >>> print(f"Downloaded to: {file_path}")
        """

        if not song_url or not song_url.strip():
            raise ValidationError("Song URL cannot be empty")
        if download_path is None:
            download_path = os.path.join(os.getcwd(), "downloads")
        
        Path(download_path).mkdir(parents=True, exist_ok=True)
        webhook_url = f"https://n8n.cubable.date/webhook/songbox?song_url={quote(song_url)}"
        try:
            with httpx.Client(follow_redirects=True, timeout=60.0) as download_client:
                with download_client.stream("GET", webhook_url) as response:
                    response.raise_for_status()
                    
                    filename = None
                    if "content-disposition" in response.headers:
                        content_disposition = response.headers["content-disposition"]
                        if "filename=" in content_disposition:
                            filename = content_disposition.split("filename=")[1].strip('"')
                    
                    if not filename:
                        final_url = str(response.url)
                        if "/" in final_url:
                            filename = final_url.split("/")[-1]
                            if "?" in filename:
                                filename = filename.split("?")[0]
                        
                        if not filename or "." not in filename:
                            filename = "song.mp3"
                    
                    if not any(filename.lower().endswith(ext) for ext in [".mp3", ".m4a", ".wav", ".flac"]):
                        filename += ".mp3"
                    
                    file_path = os.path.join(download_path, filename)
                    
                    total_size = int(response.headers.get("content-length", 0))
                    downloaded_size = 0
                    
                    print(f"Downloading: {filename}")
                    if total_size > 0:
                        print(f"File size: {total_size / (1024*1024):.2f} MB")
                    
                    with open(file_path, "wb") as file:
                        for chunk in response.iter_bytes(chunk_size=8192):
                            if chunk:
                                file.write(chunk)
                                downloaded_size += len(chunk)
                                
                                if total_size > 0:
                                    progress = (downloaded_size / total_size) * 100
                                    bar_length = 50
                                    filled_length = int(bar_length * downloaded_size // total_size)
                                    bar = "█" * filled_length + "-" * (bar_length - filled_length)
                                    print(f"\r[{bar}] {progress:.1f}% ({downloaded_size / (1024*1024):.2f}/{total_size / (1024*1024):.2f} MB)", end="")
                                else:
                                    print(f"\rDownloaded: {downloaded_size / (1024*1024):.2f} MB", end="")
                    
                    print(f"\nDownload completed: {file_path}")
                    return file_path
                    
        except httpx.HTTPStatusError as e:
            raise Exception(f"Failed to download song: HTTP {e.response.status_code}")
        except httpx.RequestError as e:
            raise Exception(f"Failed to download song: {str(e)}")
        except Exception as e:
            raise Exception(f"Download failed: {str(e)}")