"""User client for songbox API"""

from typing import Dict, Any, List, Optional
from .exceptions import AuthenticationError, NotFoundError


class UserClient:
    """Client for handling user-related operations with the songbox API.
    
    This client handles user profile information, following, and other user operations.
    """
    
    def __init__(self, client):
        self._client = client
    
    def get_me(self) -> Dict[str, Any]:
        """Get current user's profile information.
        
        Returns:
            User profile data including id, username, email, subscription info
            
        Raises:
            AuthenticationError: If not authenticated
            
        Example:
            >>> user_info = client.user.get_me()
            >>> print(f"Username: {user_info['username']}")
            >>> print(f"Subscription: {user_info['subscription']}")
        """
        try:
            response = self._client.request(
                method="GET",
                endpoint="/v1/me",
                require_auth=True
            )
            
            return response
            
        except Exception as e:
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise AuthenticationError("Authentication required or token expired")
            raise
    
    def get_following(self) -> List[Dict[str, Any]]:
        """Get list of users/artists that the current user is following.
        
        Returns:
            List of followed users/artists
            
        Raises:
            AuthenticationError: If not authenticated
            
        Example:
            >>> following = client.user.get_following()
            >>> for user in following:
            ...     print(f"Following: {user.get('name', 'Unknown')}")
        """
        try:
            response = self._client.request(
                method="GET",
                endpoint="/v1/me/following",
                require_auth=True
            )
            
            # Handle empty response (API returns {} when no following)
            if isinstance(response, dict) and not response:
                return []
            
            # If response is a list, return it
            if isinstance(response, list):
                return response
            
            # If response has a 'data' or 'following' key, return that
            if isinstance(response, dict):
                return response.get('data', response.get('following', []))
            
            return []
            
        except Exception as e:
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise AuthenticationError("Authentication required or token expired")
            raise
    
    def get_user_profile(self, user_id: str) -> Dict[str, Any]:
        """Get profile information for a specific user.
        
        Args:
            user_id: ID of the user to get profile for
            
        Returns:
            User profile data
            
        Raises:
            AuthenticationError: If not authenticated
            NotFoundError: If user not found
            
        Example:
            >>> profile = client.user.get_user_profile("12345")
            >>> print(f"User: {profile['username']}")
        """
        try:
            response = self._client.request(
                method="GET",
                endpoint=f"/v1/users/{user_id}",
                require_auth=True
            )
            
            return response
            
        except Exception as e:
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise AuthenticationError("Authentication required or token expired")
            elif "404" in str(e) or "not found" in str(e).lower():
                raise NotFoundError(f"User with ID {user_id} not found")
            raise
    
    def follow_user(self, user_id: str) -> bool:
        """Follow a user or artist.
        
        Args:
            user_id: ID of the user/artist to follow
            
        Returns:
            True if successful
            
        Raises:
            AuthenticationError: If not authenticated
            NotFoundError: If user not found
            
        Example:
            >>> success = client.user.follow_user("12345")
            >>> if success:
            ...     print("Successfully followed user")
        """
        try:
            response = self._client.request(
                method="POST",
                endpoint=f"/v1/users/{user_id}/follow",
                require_auth=True
            )
            
            # Check if the response indicates success
            return response.get('success', True)
            
        except Exception as e:
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise AuthenticationError("Authentication required or token expired")
            elif "404" in str(e) or "not found" in str(e).lower():
                raise NotFoundError(f"User with ID {user_id} not found")
            raise
    
    def unfollow_user(self, user_id: str) -> bool:
        """Unfollow a user or artist.
        
        Args:
            user_id: ID of the user/artist to unfollow
            
        Returns:
            True if successful
            
        Raises:
            AuthenticationError: If not authenticated
            NotFoundError: If user not found
            
        Example:
            >>> success = client.user.unfollow_user("12345")
            >>> if success:
            ...     print("Successfully unfollowed user")
        """
        try:
            response = self._client.request(
                method="DELETE",
                endpoint=f"/v1/users/{user_id}/follow",
                require_auth=True
            )
            
            # Check if the response indicates success
            return response.get('success', True)
            
        except Exception as e:
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise AuthenticationError("Authentication required or token expired")
            elif "404" in str(e) or "not found" in str(e).lower():
                raise NotFoundError(f"User with ID {user_id} not found")
            raise
    
    def update_profile(self, **kwargs) -> Dict[str, Any]:
        """Update current user's profile information.
        
        Args:
            **kwargs: Profile fields to update (e.g., username, email)
            
        Returns:
            Updated profile data
            
        Raises:
            AuthenticationError: If not authenticated
            
        Example:
            >>> updated = client.user.update_profile(username="newname")
            >>> print(f"Updated username: {updated['username']}")
        """
        try:
            response = self._client.request(
                method="PUT",
                endpoint="/v1/me",
                json=kwargs,
                require_auth=True
            )
            
            return response
            
        except Exception as e:
            if "401" in str(e) or "unauthorized" in str(e).lower():
                raise AuthenticationError("Authentication required or token expired")
            raise